"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normRules = normRules;
exports.default = rulesRunner;

var _actions = require("./actions");

var _actions2 = _interopRequireDefault(_actions);

var _deepcopy = require("deepcopy");

var _deepcopy2 = _interopRequireDefault(_deepcopy);

var _deepEqual = require("deep-equal");

var _deepEqual2 = _interopRequireDefault(_deepEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function doRunRules(engine, formData, schema) {
  var uiSchema = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var extraActions = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

  var schemaCopy = (0, _deepcopy2.default)(schema);
  var uiSchemaCopy = (0, _deepcopy2.default)(uiSchema);
  var formDataCopy = (0, _deepcopy2.default)(formData);

  var res = engine.run(formData).then(function (events) {
    events.forEach(function (event) {
      return (0, _actions2.default)(event, formDataCopy, schemaCopy, uiSchemaCopy, extraActions);
    });
  });

  return res.then(function () {
    return {
      schema: schemaCopy,
      uiSchema: uiSchemaCopy,
      formData: formDataCopy
    };
  });
}

function normRules(rules) {
  return rules.sort(function (a, b) {
    if (a.order === undefined) {
      if (b.order !== undefined) {
        return 1;
      } else {
        return 0;
      }
    }
    if (b.order === undefined) {
      return -1;
    }
    return a.order - b.order;
  });
}

function rulesRunner(schema, uiSchema, rules, Engine, extraActions) {
  var engine = new Engine([], schema);
  normRules(rules).forEach(function (rule) {
    return engine.addRule(rule);
  });

  return function (formData) {
    if (formData === undefined || formData === null) {
      return Promise.resolve({ schema: schema, uiSchema: uiSchema, formData: formData });
    }

    return doRunRules(engine, formData, schema, uiSchema, extraActions).then(function (conf) {
      if ((0, _deepEqual2.default)(conf.formData, formData)) {
        return conf;
      } else {
        return doRunRules(engine, conf.formData, schema, uiSchema, extraActions);
      }
    });
  };
}