"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.findRelUiSchema = findRelUiSchema;
var OBJECT = exports.OBJECT = "object";

var isDevelopment = exports.isDevelopment = function isDevelopment() {
  return process.env.NODE_ENV !== "production";
};

var toArray = exports.toArray = function toArray(field) {
  if (Array.isArray(field)) {
    return field;
  } else {
    return [field];
  }
};

var toError = exports.toError = function toError(message) {
  if (isDevelopment()) {
    throw new ReferenceError(message);
  } else {
    console.error(message);
  }
};

var fetchRefSchema = exports.fetchRefSchema = function fetchRefSchema(ref, schema) {
  if (ref.startsWith("#/")) {
    ref.substr(2).split("/");
    return ref.substr(2).split("/").reduce(function (schema, field) {
      return schema[field];
    }, schema);
  } else if (schema.properties[ref] && schema.properties[ref].type === OBJECT) {
    return schema.properties[ref];
  } else {
    toError("Only local references supported at this point");
    return undefined;
  }
};

var toRefSchema = exports.toRefSchema = function toRefSchema(field, _ref) {
  var properties = _ref.properties;

  if (properties[field]) {
    if (properties[field]["$ref"]) {
      return properties[field]["$ref"];
    } else if (properties[field].items && properties[field].items["$ref"]) {
      return properties[field].items["$ref"];
    } else if (properties[field].type === OBJECT) {
      return field;
    }
  }
  return undefined;
};

/**
 * Find relevant schema for the field
 * @returns { field: "string", schema: "object" } relevant field and schema
 */
var findRelSchemaAndField = exports.findRelSchemaAndField = function findRelSchemaAndField(field, schema) {
  var separator = field.indexOf(".");
  if (separator === -1) {
    return { field: field, schema: schema };
  }

  var parentField = field.substr(0, separator);
  var refSch = toRefSchema(parentField, schema);
  if (!refSch) {
    toError("Failed to find " + refSch);
    return { field: field, schema: schema };
  }

  var refSchema = fetchRefSchema(refSch, schema);
  if (refSchema) {
    return findRelSchemaAndField(field.substr(separator + 1), refSchema);
  }

  toError("Failed to retrieve " + refSchema + " from schema");
  return { field: field, schema: schema };
};

function findRelUiSchema(field, uiSchema) {
  var separator = field.indexOf(".");
  if (separator === -1) {
    return uiSchema;
  }

  var parentField = field.substr(0, separator);
  var refUiSchema = uiSchema[parentField];
  if (!refUiSchema) {
    return uiSchema;
  } else {
    return findRelUiSchema(field.substr(separator + 1), refUiSchema);
  }
}